# This is needed for creating data heirachy's for dates to make dates work correctly
import uuid, os


def create_date_hr(col_name, dataset_name, report_name, dashboard_path):

	# create new file id
	FILE_ID = str(uuid.uuid4())

	# create new table id
	TABLE_ID = str(uuid.uuid4())

	date_template_path = os.path.join(dashboard_path, "python_resources/LocalDateTable_FILE_ID.tmdl" )
	date_heirarchy_path = os.path.join(dashboard_path, f"{report_name}.SemanticModel/definition/tables/LocalDateTable_{FILE_ID}.tmdl" )

	with open(date_heirarchy_path, "w") as date_hr:
		with open(date_template_path, "r") as date_template:
			# read line by line and replace
			for line in date_template:

				# loop through file and replace placeholders with values
				line = line.replace("FILE_ID", FILE_ID)
				line = line.replace("TABLE_ID", TABLE_ID)

				# create a random new UIID for each column
				line = line.replace("COL_ID", str(uuid.uuid4()))

				# replace data source
				line = line.replace("DATA_SOURCE", f"'{dataset_name}'[{col_name}]" )

				# write back to file
				date_hr.write(line)



	# return the FILE_ID for later use
	return(FILE_ID)



		











#background_image
import os, shutil, json

def add_background_img(dashboard_path, page_id, img_path, alpha, scaling_method = "Fit"):

	'''



	'''

	if typ(alpha) is not int:
		raise TypeError("alpha (the transparency value) must be an integer between 1-100")

	if alpha is > 100 or < 0:
		raise ValueError("alpha (the transparency value) must be an integer between 1-100")

	# file paths
	report_name = os.path.basename(dashboard_path)
	img_name = os.path.basename(img_path)

	report_folder = os.path.join(dashboard_path, f'{report_name}.Report')
	definitions_folder = os.path.join(report_folder, "definition")

	page_json_path = os.path.join(definitions_folder, f"pages/{page_id}/page.json")
	report_json_path = os.path.join(definitions_folder, "report.json")


	registered_resources_folder = os.path.join(report_folder, "StaticResources/RegisteredResources")

	# This is the location of the image within the dashboard
	registered_img_path = os.path.join(registered_resources_folder, img_name)


	# Upload image to dashboard's registered resources ---------------------------------------------------

	# create registered resources folder if it doesn't exist
	if not os.path.exists(registered_resources_folder):
		os.makedirs(registered_resources_folder)

	# move image to registered resources folder
	shutil.copy(img_path, registered_img_path)


	# add new registered resource (the image) to report.json ----------------------------------------------
	with open(report_json_path,'r') as file:
		report_json = json.load(file)


	# add the image as an item to the registered resources items list
	for dict in report_json["resourcePackages"]:
		if dict["name"] == "RegisteredResources":
			dict["items"].append(
				                  {
                                    "name": img_name,
                                    "path": img_name,
                                    "type": "Image"
                                   }   
        	                    )



	#print(report_json)

   
	# write to file
	with open(report_json_path,'w') as file:
		json.dump(report_json, file, indent = 2)




	# Add image to page -------------------------------------------------------------------------------
	with open(page_json_path,'r') as file:
		page_json = json.load(file)


	# add the image to the page's json
	page_json["objects"].update(

		    "background": [
      {
        "properties": {
          "image": {
            "image": {
              "name": {
                "expr": {
                  "Literal": {
                    "Value": f"'{img_name}'"
                  }
                }
              },
              "url": {
                "expr": {
                  "ResourcePackageItem": {
                    "PackageName": "RegisteredResources",
                    "PackageType": 1,
                    "ItemName": img_name
                  }
                }
              },
              "scaling": {
                "expr": {
                  "Literal": {
                    "Value": f"'{scaling_method}'"
                  }
                }
              }
            }
          },
          "transparency": {
            "expr": {
              "Literal": {
                "Value": f"{alpha}D"
              }
            }
          }
        }
      }
    ]


		)
	

	

   
	# write to file
	with open(page_json_path,'w') as file:
		json.dump(page_json, file, indent = 2)






	#blorg/blorg.Report/StaticResources/RegisteredResources/Taipei_skyline_at_sunset_201506895591283012392.jpg




report_name = "blorg"
report_location = f"C:/Users/rps1303/PBI_projects"

dashboard_path = f"{report_location}/{report_name}"

add_background_img(dashboard_path = dashboard_path, page_id = "page2", img_path = "C:/Users/rps1303/Downloads/Taipei_skyline_at_sunset_20150607.jpg", alpha = 100)



import os, re, shutil

from pathlib import Path
from importlib import resources


import PBI_dashboard_creator.update_diagramLayout as PBI_DL
import PBI_dashboard_creator.update_model_file as PBI_model              # internal function to add data to model.tmdl



def add_tmdl_dataset(dashboard_path, data_path = None, add_default_datetable = True):
	
	'''



	'''

	if data_path is not None and add_default_datetable is True:
		raise ValueError("If you are providing a path to a tmdl dataset, the add_default_datetable argument can't be set to True")	
	




	# file paths
	report_name = os.path.basename(dashboard_path)

	semantic_model_folder = os.path.join(dashboard_path, f'{report_name}.SemanticModel' )
	definitions_folder = os.path.join(semantic_model_folder, "definition")
	tables_folder = os.path.join(definitions_folder, 'tables')


	# create tables folder if it doesn't already exist
	if not os.path.exists(tables_folder):
		os.makedirs(tables_folder)


	# check to see if we're importing the default DateTable.tmdl file
	# if the add default datetable argument is true
	# we'll use the path to the default datetable instead of the user supplied data_path
	# for the path to the datable we're going to add
	if add_default_datetable and data_path is None:

		# define the path we'll move the table to
		tmdl_dataset_path = os.path.join(tables_folder, "DateTable.tmdl")
		dataset_name = "DataTable"
		

		data_path = str(resources.files("PBI_dashboard_creator.dashboard_resources.python_resources").joinpath("DateTable.tmdl"))


		# copy date table from package resources to table folder
		shutil.copy(data_path, tmdl_dataset_path)

		# change the data_path to be the newly copied file
		data_path = tmdl_dataset_path

	else:
		# define the path we'll move the table to
		tmdl_dataset_path = os.path.join(tables_folder, os.path.basename(data_path))

		# dateset_name
		# extract the dataset name from the tmdl file's path
		# extract bits of names for later
		path_end = os.path.basename(data_path)
		split_end = os.path.splitext(path_end)

		dataset_name = split_end[0]

		# otherwise we move the tmdl file defined by the user to the tables folder
		# add the new tmdl file to the tables folder
		shutil.move(data_path, tmdl_dataset_path)





	# dateset_name -----------------------------------------------------------------------------------------------
	# read the whole table.tmdl file in andmake it a giant blob for regex
	file_content = ""

	with open(data_path) as file: 

		# list comprehension
		# all lines have all the white spaces and \n and \t striped 
		# They're then joined together using the .join function and ~ as a seperator
		file_content = "~".join(re.sub('\t?', '', line).rstrip() for line in file)


	# pull out just the dataset_id using regex
	m = re.search("(?<=lineageTag: ).*?(?=~~column)", file_content )


	dataset_id = m.group(0)




	# update the diagramLayout file to include the new date table\
	PBI_DL.update_diagramLayout(dashboard_path = dashboard_path, dataset_name = dataset_name, dataset_id = dataset_id)

	# update the model.tmdl file to include the new datetable
	PBI_model.update_model_file(dashboard_path = dashboard_path, dataset_name = dataset_name)







import json, os

def update_diagramLayout(dashboard_path, dataset_name, dataset_id):

	'''
	This is an internal function to add a dataset to the diagramLayout file when a new dataset is added. 

	:param str dashboard_path The path to the top level folder where you store all the report's files.
	:param str dataset_name The name of the dataset you are adding
	:param str dataset_id The unique uuid that microsoft uses for the dataset. This is generated automatically within the add_data functions. 
	return None




	'''


	# file paths   --------------------------------------------------------
	report_name = os.path.basename(dashboard_path)
	semantic_model_folder = os.path.join(dashboard_path, f'{report_name}.SemanticModel' )
	definitions_folder = os.path.join(semantic_model_folder, "definition")

	diagram_layout_path = os.path.join(semantic_model_folder, 'diagramLayout.json')






	with open(diagram_layout_path,'r') as file:
		diagram_layout = json.load(file)


	# add all this junk to describe the table's "nodes"
	diagram_layout["diagrams"][0]["nodes"].append( 
        {
          "location": {
            "x": 0,
            "y": 0
          },
          "nodeIndex": dataset_name,
          "nodeLineageTag": dataset_id,
          "size": {
            "height": 300,
            "width": 234
          },
          "zIndex": 0
        }
      )

   
	# write to file
	with open(diagram_layout_path,'w') as file:
		json.dump(diagram_layout, file, indent = 2)





# modify modil tmdl
import re, shutil, os

def update_model_file(dashboard_path, dataset_name):

	'''
	This is an internal function to add a dataset to the model.tmdl file when a new dataset is added. 
	It assumes you want the new dataset to be loaded last. 

	:param str dashboard_path The path to the top level folder where you store all the report's files.
	:param str dataset_name The name of the dataset you are adding
	return None



	'''


	# file paths
	report_name = os.path.basename(dashboard_path)

	semantic_model_folder = os.path.join(dashboard_path, f'{report_name}.SemanticModel' )
	definitions_folder = os.path.join(semantic_model_folder, "definition")
	model_path = os.path.join(definitions_folder, 'model.tmdl')
	temp_model_path = os.path.join(dashboard_path, 'model2.tmdl')


	# loop through all the lines in the model file
	# to find that part that lists the order of the datasets

	with open(temp_model_path, 'w') as tmp:
		with open(model_path, "r") as file:
			for line in file.readlines():

				# check to see if the line is the one we want
				m = re.search("(?<=annotation PBI_QueryOrder = ).*", line)

				# if it is, read the list of datasets and append a new one in

				if m is not None:
					# execute the tmdl code to make a python list

					# execute the code (including local and global scopes)
					# source: https://stackoverflow.com/questions/41100196/exec-not-working-inside-function-python3-x
					exec(f'query_order_list = {m.group(0)}', locals(), globals())

					# add the dataset using python method then write back  to line
					query_order_list.append(dataset_name)
					line = f'annotation PBI_QueryOrder = {query_order_list}\n'

					# write back the line to a temporary file

				tmp.write(line)

			# append the dataset name at the end of the file
			tmp.write(f"\n\nref table {dataset_name}")


  # Replace the model file with the temp file we created
	shutil.move(temp_model_path, model_path)





import os, json

def add_new_page(dashboard_path, page_name):

	# file paths
	report_name = os.path.basename(dashboard_path)
	pages_folder = os.path.join(dashboard_path, f'{report_name}.Report/definition/pages' )
	pages_json_path = os.path.join(pages_folder, "pages.json")

	# determine number of pages
	with open(pages_json_path,'r') as file:
		pages_list = json.load(file)

		# determine number of pages
		n_pages = len(pages_list["pageOrder"])

		# create a new page id based on existing number of pages
		new_page_id = f"page{n_pages + 1}"

		# add the new page id to the pageOrder list
		pages_list["pageOrder"].append(new_page_id)
    
  
	# write to file
	with open(pages_json_path,'w') as file:
		json.dump(pages_list, file, indent = 2)

	
	# create a folder for the new page
	new_page_folder = os.path.join(pages_folder, new_page_id)
	new_page_json_path = os.path.join(new_page_folder, "page.json")
	os.makedirs(new_page_folder)

	# create a new json file for the new page
	new_page_json = {"$schema": "https://developer.microsoft.com/json-schemas/fabric/item/report/definition/page/1.2.0/schema.json",
	                  "name": new_page_id,
	                  "displayName": page_name,
	                  "displayOption": "FitToPage",
	                  "height": 720,
	                  "width": 1280,
					  "objects":{}}

	# write to file
	with open(new_page_json_path, "w") as file:
		json.dump(new_page_json, file, indent = 2)

